#!/usr/bin/env python2

import sys, getopt, os, sha, rfc822, shutil, xml.sax, string, re

import xmlparsers

def help():
    """Print help information and quit"""

    mesg = """Options:
    createlist -s sourcedir -d outputdir -o outputfile

    sourcedir - name of the directory containing the PEP241 descriptions and
                packages.  Packages assumed to be in sourcedir/packages and
                descriptions in sourcedir/descriptions
    outputdir - name of the directory to create the directory structure in
    outputfile - name of the file to write the packagelist in"""
    sys.stdout.write(mesg);
    sys.exit(1);

def convertPEP(inFile, outFile, sourceDir, outputDir):
    """Converts PEP241 spec found in inFile to an entry for ciphon"""

    try:
        spec = rfc822.Message(inFile)
    except:
        return 1
    name = spec.getheader("Name")
    version = spec.getheader("Version")
    description = spec.getheader("Description")
    authorEmail = []
    for x in spec.getallmatchingheaders('Author-email'):
        authorEmail.append((x.split('<')[0], x.split[1][:-1]))
    license = spec.getheader('License')
    url = spec.getheader('Home-page')
    if not url:
        url = "Unavailable"
    filename = spec.getheader('Filename')
    dependencies = {}
    for x in spec.getallmatchingheaders('Dependency'):
        dep = x.split()
        temp = {}
        temp['name'] = x[0]
        temp['type'] = 'REQUIRE'
        temp['version'] = x[2]
        if x[1] == '>=' or x[1] == '>':
            temp['version-type'] = 'MINIMUM'
        else:
            temp['version-type'] = 'MAXIMUM'
        dependences[x[0]] = temp

    hash = sha.new(name)
    hashDir = hash.hexdigest()[0]

    packageName = "%s-%s.tar.gz" % (name, version)
    packageLocation = os.path.join(sourceDir, 'packages')
    packageDestination = os.path.join(outputDir, hashDir)

    if not (name or version or authorEmail):
        sys.stderr.write("Name, version, or author's email is not specified\n")
        return 1

    try:
        size = os.stat(os.path.join(packageLocation, packageName))[6]
    except OSError:
        sys.stderr.write("Error getting size: %s/%s\n" %
                                         (packageLocation, packageName))
        return 1

    try:
        if not os.access(packageDestination, os.F_OK|os.W_OK) == 1:
            os.makedirs(packageDestination, 0755)
    except OSError:
        sys.stderr.write("Error creating directory: %s\n" % packageDestination)
        return 1


    try:
        shutil.copyfile(os.path.join(packageLocation, packageName),
                        os.path.join(packageDestination, packageName))
    except:
        sys.stderr.write("Error copying %s to %s\n" %
                         (packageLocation, packageDestination))
        return 1


    directory = hashDir
    outFile.write("<package>\n")
    outFile.write("<name>%s</name>\n" % name)
    for (authorName, authorEmail) in authorEmail:
        outFile.write('<author name="%s" email="%s" />' % (authorName,
                                                           authorEmail))
    outFile.write("<version>%s</version>\n" % version)
    outFile.write("<license>%s</license>\n" % license)
    outFile.write("<description>%s</description>\n" % description)
    outFile.write("<url>%s</url>\n" % url)
    outFile.write("<filename>%s</filename>\n" % filename)
    outFile.write("<dependencies>\n")
    for dependency in dependencies.keys():
        dep = dependencies[dependency]
        outFile.write('<dependency type="%s">\n' % dep['type'])
        outFile.write('<name>%s</name>\n' % dep['name'])
        outFile.write('<version type="%s">%s</version>\n' % (dep['version-type'],
                                                             dep['version']))
        outFile.write('</dependency')
    outFile.write("</dependencies>\n")
    outFile.write("</package>\n")
    return 0

def convertXML(inFile, outFile, sourceDir, outputDir):
    """Converts XML spec found in inFile to an entry for ciphon"""

    package = ''
    try:
        specParser = xmlparsers.PackageReader()
        xml.sax.parse(inFile, specParser)
        temp = specParser.getPackageList()
        package = temp[temp.keys()[0]] # assume only one package in spec
    except:
        return 1

    name = package['name']
    version = package['version']
    filename = package['filename']

    hash = sha.new(name)
    hashDir = hash.hexdigest()[0]

    packageName = "%s-%s.tar.gz" % (filename, version)
    packageLocation = os.path.join(sourceDir, 'packages')
    packageDestination = os.path.join(outputDir, hashDir)

    try:
        size = os.stat(os.path.join(packageLocation, packageName))[6]
    except OSError:
        sys.stderr.write("Error getting size: %s/%s\n" %
                                         (packageLocation, packageName))
        return 1

    try:
        if not os.access(packageDestination, os.F_OK|os.W_OK) == 1:
            os.makedirs(packageDestination, 0755)
    except OSError:
        sys.stderr.write("Error creating directory: %s\n" % packageDestination)
        return 1


    try:
        shutil.copyfile(os.path.join(packageLocation, packageName),
                        os.path.join(packageDestination, packageName))
    except:
        sys.stderr.write("Error copying %s to %s\n" %
                         (packageLocation, packageDestination))
        return 1


    directory = hashDir

    inFile.seek(0)
    package = string.join(inFile.readlines(), "")
    xmlRegex = r'(<package>.*?</filename>).*?(<dependencies.*?</package>)'
    matches = re.search(xmlRegex,  package, re.S|re.M)
    outFile.write("%s<size>%d</size>\n<path>%s</path>\n%s" % (matches.group(1),
                                                              size,
                                                              directory,
                                                              matches.group(2)))
                                                              
    return 0


presentdir = os.getcwd()
try:
    optionList, arguments = getopt.getopt(sys.argv[1:], 'b:d:o:s:')
except getopt.error, error:
    sys.stderr.write("Unrecognized option %s\n" % error)
    help()
    sys.exit(1)

sourceDirectory = ''
sourceDirectory = ''
baseDirectory = ''
outputFile = ''
parameters = {}

for (option, text) in optionList:
    if option == '-s':
        sourceDirectory = text
    if option == '-d':
        outputDirectory = text
    elif option == '-o':
        outputFile = text

if sourceDirectory == '' or outputFile == '':
    help()

if os.access(outputDirectory, os.F_OK):
    sys.stderr.write("Output directory exists, exiting!\n")
    sys.exit(1)

if os.access(outputFile, os.W_OK):
    sys.stderr.write("Can't write to output file exiting\n")
    sys.exit(1)

try:
    fileList = os.listdir("%s/descriptions" % sourceDirectory)
except OSError, error:
    sys.stdout.write(error.args[1] + "\n")
    sys.stdout.write("Quiting...\n")
    sys.exit(1)

try:
    outFile = open(outputFile, 'w')
    outFile.write('<packagelist>')
except IOError, error:
    sys.stderr.write(error.args[1] + "\n")
    sys.stdout.write("Quiting...\n")
    sys.exit(1)

for specFile in filter(lambda x: x[-4:] == 'spec', fileList):
    print "pep processing %s" % specFile
    try:
        inFile = open("%s/descriptions/%s" % (sourceDirectory, specFile), 'r')
    except IOError, error:
        sys.stderr.write(error.args[1] + "\n")
        sys.stdout.write("Quiting...\n")
        sys.exit(1)

    if convertPEP(inFile, outFile, sourceDirectory, outputDirectory):
        sys.stderr.write("Error processing %s, skipping...\n" % specFile);

    inFile.close()

for specFile in filter(lambda x: x[-3:] == 'xml', fileList):
    print "xml processing %s" % specFile
    try:
        inFile = open("%s/descriptions/%s" % (sourceDirectory, specFile), 'r')
    except IOError, error:
        sys.stderr.write(error.args[1] + "\n")
        sys.stdout.write("Quiting...\n")
        sys.exit(1)

    if convertXML(inFile, outFile, sourceDirectory, outputDirectory):
        sys.stderr.write("Error processing %s, skipping...\n" % specFile);

    inFile.close()
    
outFile.write('</packagelist>')
outFile.close()
